-- MMEagleFunctions
-- Author: Magnus
-- DateCreated: 10/7/2018 9:15:44 PM
--------------------------------------------------------------
include("MMEagleGlobals.lua") 
print("gNumNaturalWonders")
print(gNumNaturalWonders)
local tEagleTeams = {}

----------------------------------------
--UpdateDrugStoreBonuses(iPlayer)
--Goes through a player's cities, deciding if that city needs the extra gold and production from drugstores or not
----------------------------------------
function UpdateDrugStoreBonuses(iPlayer)
	print("UpdateDrugStoreBonuses firing")
	local pPlayer = Players[iPlayer]
	for pCity in pPlayer:Cities() do
		pCity:SetNumRealBuilding(gEagleGold, 0)	
		--Check for Drugstores
		if pCity:IsHasBuilding(gDrugstore) then
			--If the city has a drugstore and is connected to the capital, give the bonus
			if pPlayer:IsCapitalConnectedToCity(pCity) then
				dprint("This city has a Drugstore and is connected to the capital!")
				pCity:SetNumRealBuilding(gDrugstoreBonus, 1)
			--Does the city have the Drugstore bonus, and shouldn't?
			elseif pCity:IsHasBuilding(gDrugstoreBonus) then
				dprint("This city has a Drugstore Bonus, but is not connected to the capital!")
				pCity:SetNumRealBuilding(gDrugstoreBonus, 0)
			end
		end
		--I'm going to add this check here, just so we don't have to iterate through player cities multiple times in one turn
		for k, v in pairs(tMMEagleCapitalDummies) do
			if pCity:IsHasBuilding(k) and not pCity:IsCapital() then
				dprint("This city isn't the capital, and it has a dummy building that it shouldn't")
				pCity:SetNumRealBuilding(k, 0)
			end
		end
	end
end
---------------------------------------------------
--GiveFoodBonusToAllCities
--Iterates through the passed player's cities,
--Adds iAmount of food to each city
---------------------------------------------------
function GiveFoodBonusToAllCities(iPlayer, iAmount)
	for pCity in Players[iPlayer]:Cities() do
		pCity:ChangeFood(iAmount)
	end
end
-------------------------------------------------------------------------------------
--UpdatePromotionToUnitType
--Iterates through the iPlayer's units
--If that unit matches the passed iUnitType, give or remove iPromotionType to that unit based on bGiveOrRemove
-------------------------------------------------------------------------------------
function UpdatePromotionToUnitType(iPlayer, iUnitType, iPromotionType, bGiveOrRemove)
	for pUnit in Players[iPlayer]:Units() do
		if pUnit:GetUnitType() == iUnitType then pUnit:SetHasPromotion(iPromotionType, bGiveOrRemove) end
	end
end


--------------------------------------
--GetSanctuaryProgress
--Checks for how many eighths of the world's natural wonders iPlayer has found
--------------------------------------
function GetSanctuaryProgress(iPlayer)
	--Get the amount of natural wonders found by the player
	local iNaturalWondersFound = (Players[iPlayer]:GetHappinessFromNaturalWonders() - GetHappinessFromWorkedNaturalWonders(iPlayer)) / gNaturalWonderHappiness
	dprint("Player " .. iPlayer .. " has found  " .. iNaturalWondersFound .. " Natural Wonders")
	--Split that into eighths
	local iProgress =  8 / gNumNaturalWonders
	dprint("Natural Wonders factor " ..  iProgress)
	return math.floor(iNaturalWondersFound * iProgress)
end

------------------------------------------------
--DetermineNessSanctuaryProgress
--Checks the player's progress with the UA
------------------------------------------------
function DetermineNessSanctuaryProgress(iTeam, iTech, iChange)
	local pTeam = Teams[iTeam]
	dprint("Determining Sanctuary Progress for team " .. iTeam)
	--This will be true if the function fired from the TeamSetHasTech hook
	dprint(iChange)
	if pTeam:IsMinorCiv() or iTeam == 63 then return end
	local iEagleland = nil
	for k, v in pairs(tEagleTeams[iTeam]) do
		local pPlayer = Players[k]
		if pPlayer:GetCivilizationType() == gEagleland then
			--It's possible for there to be multiple Eaglelands on the same team
			local pPlayer = Players[k]
			local iProgress = GetSanctuaryProgress(k)
			--Check if the player has a received their bonus for the amount of NWs found yet
			--We used to use straight-up use dummy techs to track UA progress, but that proved problematic
			--However, seeing as dummy techs popups sufficiently suit our needs, I left them in -- the script simply gives the player
			--the dummy tech to cause the pop-up, and then prompty takes it away 
			--In other words, the techs themselves DON'T ACTUALLY DO ANYTHING!
			--For now, at least. We'll have to see if that changes
			local iBonuses = g_tSavedData.GetValue("Player " .. k .. " Sanctuaries Discovered") or 0
			dprint("iProgress " .. iProgress)
			dprint("iBonuses " .. iBonuses)
	
			if iProgress == iBonuses then
				dprint("Ness has made no new progress yet. Abort!")
				return
			else
				--The player is ready for the next bonus, give it to them
				dprint("Ness is ready for another bonus")
				local i = iBonuses + 1
				--Find the next bonus the player needs
				--while i < iBonuses do i = i + 1 end
				local pCapital = pPlayer:GetCapitalCity()
				if i == 1 then
					--2 psychics appear, a golden age begins.
					--PK Fire and PK Freeze are used more often
					dprint("1/8")
					local pUnit = pPlayer:InitUnit(GameInfoTypes.UNIT_MM_PSYCHIC, pCapital:GetX(), pCapital:GetY(), UNITAI_ATTACK)
					pUnit:JumpToNearestValidPlot()
					pUnit = pPlayer:InitUnit(GameInfoTypes.UNIT_MM_PSYCHIC, pCapital:GetX(), pCapital:GetY(), UNITAI_ATTACK)
					pUnit:JumpToNearestValidPlot()
					pPlayer:ChangeGoldenAgeTurns(10 * (GameInfo.GameSpeeds[Game:GetGameSpeedType()].GoldenAgePercent / 100)) --We may want to DB.Query in the future
						--GameInfo can sometimes miss changes made to Gamespeeds via other mods. This might be true for other tables, but I'm not certain
					pTeam:SetHasTech(tNessProgress[1], true)
					pTeam:SetHasTech(tNessProgress[1], false)
					--if iChange then
					--	dprint("Fixing tech view")
					--	pTeam:SetHasTech(iTech, false)
					--	pTeam:SetHasTech(iTech, true)
					--end
					g_tSavedData.SetValue("Player " .. k .. " Sanctuaries Discovered", 1)
				elseif i == 2 then
					--+2 Happiness
					--PK Thunder and Lifeup are used more often 
					dprint("2/8")
					pCapital:SetNumRealBuilding(gNessHappiness, 1)
					pTeam:SetHasTech(tNessProgress[2], true)
					pTeam:SetHasTech(tNessProgress[2], false)
					--if iChange then
					--	dprint("Fixing tech view")
					--	pTeam:SetHasTech(iTech, false)
					--	pTeam:SetHasTech(iTech, true)
					--end
					g_tSavedData.SetValue("Player " .. k .. " Sanctuaries Discovered", 2)
				elseif i == 3 then
					--Psychics are stronger
					--Lifeup heals more, Shield is used more often
					dprint("3/8")
					pTeam:SetHasTech(tNessProgress[3], true)
					pTeam:SetHasTech(tNessProgress[3], false)
					--if iChange then
					--	dprint("Fixing tech view")
					--	pTeam:SetHasTech(iTech, false)
					--	pTeam:SetHasTech(iTech, true)
					--end
					g_tSavedData.SetValue("Player " .. k .. " Sanctuaries Discovered", 3)
				elseif i == 4 then
					--Food boost in all cities
					--Psychics may use hypnosis 
					dprint("4/8")
					GiveFoodBonusToAllCities(k, 60 * (GameInfo.GameSpeeds[Game:GetGameSpeedType()].GrowthPercent / 100))
					pTeam:SetHasTech(tNessProgress[4], true)
					pTeam:SetHasTech(tNessProgress[4], false)
					--if iChange then
					--	dprint("Fixing tech view")
					--	pTeam:SetHasTech(iTech, false)
					--	pTeam:SetHasTech(iTech, true)
					--end
					g_tSavedData.SetValue("Player " .. k .. " Sanctuaries Discovered", 4)
				elseif i == 5 then
					--Psychics are stronger
					--Psychics will use stronger shield (reflects damage), may teleport
					dprint("5/8")
					UpdatePromotionToUnitType(k, gMMPsychic, gTeleportI, true)
					pTeam:SetHasTech(tNessProgress[5], true)
					pTeam:SetHasTech(tNessProgress[5], false)
					--if iChange then
					--	dprint("Fixing tech view")
					--	pTeam:SetHasTech(iTech, false)
					--	pTeam:SetHasTech(iTech, true)
					--end
					g_tSavedData.SetValue("Player " .. k .. " Sanctuaries Discovered", 5)
				elseif i == 6 then
					--+8 Happiness
					dprint("6/8")
					pCapital:SetNumRealBuilding(gNessHappiness, 5)
				--	pPlayer:ChangeGoldenAgeTurns(30 * (GameInfo.GameSpeeds[Game:GetGameSpeedType()].GoldenAgePercent / 100))
					pTeam:SetHasTech(tNessProgress[6], true)
					pTeam:SetHasTech(tNessProgress[6], false)
					--if iChange then
					--	dprint("Fixing tech view")
					--	pTeam:SetHasTech(iTech, false)
					--	pTeam:SetHasTech(iTech, true)
					--end
					g_tSavedData.SetValue("Player " .. k .. " Sanctuaries Discovered", 6)
				elseif i == 7 then
					--WLTKD in all cities, culture boost
					--Psychics will use hypnosis more often
					dprint("7/8")
					for pCity in pPlayer:Cities() do pCity:ChangeWeLoveTheKingDayCounter(15 * (GameInfo.GameSpeeds[Game:GetGameSpeedType()].GrowthPercent / 100)) end
					pPlayer:ChangeJONSCulture(500 * (GameInfo.GameSpeeds[Game:GetGameSpeedType()].CulturePercent / 100))
					pTeam:SetHasTech(tNessProgress[7], true)
					pTeam:SetHasTech(tNessProgress[7], false)
					--if iChange then
					--	dprint("Fixing tech view")
					--	pTeam:SetHasTech(iTech, false)
					--	pTeam:SetHasTech(iTech, true)
					--end
					g_tSavedData.SetValue("Player " .. k .. " Sanctuaries Discovered", 7)
				elseif i == 8 then
					--Psychics are stronger and may be trained normally (expensive!)
					--Very Long Golden Age, +33% Global GP generation
					--Teleport is stronger
					dprint("8/8")
					UpdatePromotionToUnitType(k, gMMPsychic, gTeleportI, false)
					UpdatePromotionToUnitType(k , gMMPsychic, gTeleportII, true)
					pPlayer:ChangeGoldenAgeTurns(50 * (GameInfo.GameSpeeds[Game:GetGameSpeedType()].GoldenAgePercent / 100))
					pCapital:SetNumRealBuilding(gNessGP, 1)
					pTeam:SetHasTech(tNessProgress[8], true)
					pTeam:SetHasTech(tNessProgress[8], false)
					--if iChange then
					--	dprint("Fixing tech view")
					--	pTeam:SetHasTech(iTech, false)
					--	pTeam:SetHasTech(iTech, true)
					--end
					g_tSavedData.SetValue("Player " .. k .. " Sanctuaries Discovered", 8)
				end
			end
		end
	end
end
GameEvents.NaturalWonderDiscovered.Add(DetermineNessSanctuaryProgress)
--GameEvents.TeamTechResearched.Add(DetermineNessSanctuaryProgress)

function DamageReflect(pUnit)
	dprint("This Psychic had a PSI Shield, reflecting damage")
	dprint("Health was " .. g_tSavedData.GetValue("Unit" .. pUnit:GetID() .. "_CurrentHealth"))
	dprint("Unit has taken " .. pUnit:GetDamage() .. " damage")
	local iPlayer = pUnit:GetOwner()
	local iDamage = g_tSavedData.GetValue("Unit" .. pUnit:GetID() .. "_CurrentHealth") - (100 - pUnit:GetDamage())
	if iDamage > 0 then
		local iUnits = GetNumNearbyEnemies(pUnit:GetX(), pUnit:GetY(), iPlayer, 2)
		if iUnits > 0 then
			iDamage = math.floor(iDamage / iUnits)
			dprint(iDamage .. "damage distributed across " .. iUnits .. " enemies")
			for k, v in pairs(tMMNessEnemies) do
				local pTargetUnit = Players[v.Player]:GetUnitByID(v.ID)
				pTargetUnit:ChangeDamage(iDamage)
				Players[pTargetUnit:GetOwner()]:AddNotification(NotificationTypes.NOTIFICATION_GREAT_PERSON_ACTIVE_PLAYER, "An enemy psyhic reflected its shield, dealing " .. iDamage .. " damage to your " .. pTargetUnit:GetName() .. " !", "Shield Reflect!", pTargetUnit:GetX(), pTargetUnit:GetY(), pTargetUnit:GetUnitType())
			end
			g_tSavedData.SetValue("Unit" .. pUnit:GetID() .. "_CurrentHealth", nil)
			tMMEnemies = {}
			tMMNessEnemies = {}
			Players[iPlayer]:AddNotification(NotificationTypes.NOTIFICATION_GREAT_PERSON_ACTIVE_PLAYER, "Your Psychic reflected their PSI Shield, dealing " .. iDamage .. " damage to " .. iUnits .. " enemies!", "Shield Reflect!", pUnit:GetX(), pUnit:GetY(), gMMPsychic)
		end
	end
end

-----------------------------  
--EagleDoTurn
--Fires at every turn start
--Does the following in order:
--	Checks Drugstore Bonuses
--	Checks UA Progress
--	Checks for Dummy Buildings
--	Updates Psychics, calls PSI attempts
-----------------------------
function EagleDoTurn(iPlayer)
	--dprint("EagleDoTurn")
	local pPlayer = Players[iPlayer]
	local iBonuses = g_tSavedData.GetValue("Player " .. iPlayer .. " Sanctuaries Discovered") or 0
	local pCapital = pPlayer:GetCapitalCity()
	if pPlayer:GetCivilizationType() == gEagleland then 
		--The capital has no bus stop, Eagleland has no extra trade route
		if not(pCapital:IsHasBuilding(gBusStop)) then
			pCapital:SetNumRealBuilding(gTradeRoute1, 0)
		end
		UpdateDrugStoreBonuses(iPlayer)
		DetermineNessSanctuaryProgress(pPlayer:GetTeam())
		local tTradeRoutes = pPlayer:GetTradeRoutes()
		dprint("Trade Routes")
		for k, v in pairs(tTradeRoutes) do
			if v.ToID == iPlayer and v.FromCity:IsHasBuilding(gBusStop) and v.Domain == DomainTypes.DOMAIN_LAND then
				dprint("Internal trade route from Bus Stop")
				local iGold = ((v.ToFood + v.ToProduction) / 100) / 2
				if v.ToCity:IsHasBuilding(gBusStop) then iGold = iGold * 2 end
				dprint("Giving " .. v.FromCityName .. " " .. iGold .. " Gold")
				v.FromCity:SetNumRealBuilding(gEagleGold, v.FromCity:GetNumRealBuilding(gEagleGold) + iGold)
			end
		end	
	--Checks to see if the capital has all the dummy buildings it should
		if iBonuses == 8 and not pCapital:IsHasBuilding(gNessGP) then
			pCapital:SetNumRealBuilding(gNessGP, 1)
			pCapital:SetNumRealBuilding(gNessHappiness, 5)
		elseif iBonuses == 6 and not pCapital:IsHasBuilding(gNessHappiness) then
			pCapital:SetNumBuilding(gNessHappiness, 5)
		elseif iBonuses == 2 and not pCapital:IsHasBuilding(gNessHappiness) then
			pCapital:SetNumBuilding(gNessHappiness, 1)
		end
	end
	if tMMPsychics[iPlayer] then
		for k, v in pairs(tMMPsychics[iPlayer]) do
			--Is the Psychic still alive?
			if pPlayer:GetUnitByID(k) then
				local pUnit = pPlayer:GetUnitByID(k)
				if pUnit then
					--Update combat strength
					EaglePsychicCreated(iPlayer, k)
					--Check if the unit used PSI shield
				end
			--	AttemptPSI(iPlayer, k)
				--Remove this index from the table, the unit here no longer exists
				else tMMPsychics[iPlayer][k] = nil
			end
		end
	end
end
GameEvents.PlayerDoTurn.Add(EagleDoTurn)

function CanConstructBusStop(iPlayer, iCity, iBuilding)
	local pPlayer = Players[iPlayer]
	if iBuilding == gBusStop then return pPlayer:IsCapitalConnectedToCity(pPlayer:GetCityByID(iCity)) 
	else return true end
end
GameEvents.CityCanConstruct.Add(CanConstructBusStop)

function OnBusStopBuilt(iPlayer, iCity, iBuilding)
	local pPlayer = Players[iPlayer]
	local pCapital = pPlayer:GetCapitalCity()
	if not pCapital then return end
	if not(iCity == pCapital:GetID() and iBuilding == gBusStop) then return end
	pCapital:SetNumRealBuilding(gTradeRoute1, 1)
end
GameEvents.CityConstructed.Add(OnBusStopBuilt)

----------------------------------------------------------------
--EscargoImprovementFinished
--Used to make the Escargo Express Man functional
----------------------------------------------------------------
function EscargoImprovementFinished(iPlayer, x, y, iImprovement)

	local pPlot = Map.GetPlot(x, y)
	local pPlayer = Players[iPlayer]
	local bPickup = iImprovement == gEscargoPickup
	local bDelivery = iImprovement == gEscargoDelivery
	local pUnit = nil
	if not(bPickup or bDelivery) then return end
	pPlot:SetImprovementType(-1)
	for i = 0, pPlot:GetNumUnits() - 1 do
		if pPlot:GetUnit(i):GetUnitType() == gEscargoExpressMain and bPickup then
			--Escargo Express Man, picking up resource
			dprint("Picking up resource")
			break
		elseif pPlot:GetUnit(i):GetUnitType() == gEscargoExpressCarry and bDelivery then
			--Escargo Express Man, completing delivery
			dprint("Dropping off resource")
			pUnit = pPlot:GetUnit(i)
			break
		end
	end
	if bPickup then
		if not pPlot:GetResourceType() then return end
		pUnit = pPlayer:InitUnit(gEscargoExpressCarry, x, y, UNITAI_WORKER)
		--Set the name of the unit to what it is carrying
		pUnit:SetName(pUnit:GetNameNoDesc() .. " Carrying " .. Locale.ConvertTextKey(GameInfo.Resources[pPlot:GetResourceType()].Description))
		pUnit:FinishMoves()
		if Players[pPlot:GetOwner()] then
			local pOwner = Players[pPlot:GetOwner()]
			if pOwner:IsMinorCiv() then
				pOwner:ChangeMinorCivFriendshipWithMajor(iPlayer, -55)
				pPlayer:AddNotification(NotificationTypes.NOTIFICATION_GENERIC, "You've stolen a resource from " .. pOwner:GetName() .. "! Influence with them has decreased by -" .. 55 .. ".", "Resource Stolen!", x, y)
			end
		end
		g_tSavedData.SetValue("Unit" .. pUnit:GetID() .. "_CarryingResource", pPlot:GetResourceType())
		g_tSavedData.SetValue("Unit" .. pUnit:GetID() .. "_ResourceQuantity", pPlot:GetNumResource())
		pPlot:SetResourceType(-1, 0)
	else
		pPlot:SetResourceType(g_tSavedData.GetValue("Unit" .. pUnit:GetID() .. "_CarryingResource"), g_tSavedData.GetValue("Unit" .. pUnit:GetID() .. "_ResourceQuantity"))
		if Players[pPlot:GetOwner()] then
			local pOwner = Players[pPlot:GetOwner()]
			if pOwner:IsMinorCiv() then
				pOwner:ChangeMinorCivFriendshipWithMajor(iPlayer, 45)
				pPlayer:AddNotification(NotificationTypes.NOTIFICATION_GENERIC, pOwner:GetName() .. " is grateful to for the gift you have bestowed upon them. Influence with them has increased by " .. 45 .. ".", "Resource Gifted!", x, y)
			end
		end
	end
end
GameEvents.BuildFinished.Add(EscargoImprovementFinished)
------------------------------------------------------------------------
--OnEagleTrainUnit
--Fires whenever an Eaglelandian city trains a unit
--If that unit is a combat unit, there is 1/128 chance of it becoming a psychic
--Chances may be higher depending how far along player is with UA
------------------------------------------------------------------------
function OnEagleTrainUnit(iPlayer, iCity, iUnit, bGold, bFaithOrCulture)
	local pPlayer = Players[iPlayer]
	if not(pPlayer:GetCivilizationType() == gEagleland) then return end 
	print("A city has trained a unit... seeing if it's valid to become a psychic...")
	local pUnit = pPlayer:GetUnitByID(iUnit)
	--The unit must be a land unit and a combat unit
	if pUnit:IsCombatUnit() and pUnit:GetDomainType() == DomainTypes.DOMAIN_LAND then
		local iChance = 64 - (g_tSavedData.GetValue("Player " .. iPlayer .. " De_MuTraining") or 0)
		--Since the AI is not as well versed at using Psychics properly, up their chances of obtaining them to counterbalance
		if not pPlayer:IsHuman() then iChance = 36 end
		--local iProgress =  8 / gNumNaturalWonders
		--local iNaturalWondersFound = pPlayer:GetHappinessFromNaturalWonders() / gNaturalWonderHappiness
		--iProgress = math.floor(iNaturalWondersFound * iProgress)
		local iProgress = g_tSavedData.GetValue("Player " .. iPlayer .. " Sanctuaries Discovered") or 0
	--	if iProgress > 0 then iChance = iChance / iProgress
		local iRoll = Game.Rand(iChance, "Ness roll") 
		dprint("iRoll " .. iRoll)
		dprint("iProgress " .. iProgress)
		--iProgress can range from 0 to 8
		--It represents every eighth of the world's natural wonders Ness has found
		--Therefore, the chances of getting a Psychic ranges from 1/128 to 9/128 (1/13), depending on your progress with the UA
		if iRoll <= iProgress then
			print("Converting this unit to a psychic!")
			local pNewUnit = pPlayer:InitUnit(GameInfoTypes.UNIT_MM_PSYCHIC, pUnit:GetX(), pUnit:GetY(), UNITAI_ATTACK)
			--We might not use Convert, but just get the old unit's experience and give it the psychic
			--pUnit:Convert(pNewUnit)
			iXP = pUnit:GetExperience()
			if g_tSavedData.GetValue("Player " .. iPlayer .. " De_MuTraining") then
				iXP = iXP + 15
			end
			pNewUnit:ChangeExperience(iXP)
			if bGold or bFaithOrCulture then pNewUnit:FinishMoves() end
			pUnit:Kill(true, -1)
			pPlayer:AddNotification(NotificationTypes.NOTIFICATION_GREAT_PERSON_ACTIVE_PLAYER, "Your " .. pUnit:GetName() .. " has shown signs of being capable of using PSI! They give their up traditional methods of fighting, and accept their role as a Psychic!", "A Psychic appears", pNewUnit:GetX(), pNewUnit:GetY(), gMMPsychic)
		end
	end
end
GameEvents.CityTrained.Add(OnEagleTrainUnit)


--------------------------------------
--PsychicCreated
--Fires when a unit is created
--If the unit has our dummy promotion, it is a new psychic
---------------------------------------
function EaglePsychicCreated(iPlayer, iUnit)
	local pPlayer = Players[iPlayer]
	local pTeam = Teams[pPlayer:GetTeam()]
	local pUnit = pPlayer:GetUnitByID(iUnit)
	if not pUnit then return end
	--if not(pPlayer:GetCivilizationType() == gEagleland) then return end
	if pUnit:GetUnitType() == gMMPsychic then
		if pUnit:IsHasPromotion(gMMPsychicD) then
			dprint("This is a new psychic")
			--Set the strength of the psychic to be 75% of the player's strongest melee unit, if > 15
			--iStrength = math.ceil(GetStrongestMilitaryUnitStrength(pPlayer, true, "UNITCOMBAT_MELEE", "UNITCOMBAT_GUN") * .75)
			--if iStrength > 15 then pUnit:SetBaseCombatStrength(iStrength) end--TODO: Be sure to include strength from the UA, once implemented
			pUnit:SetHasPromotion(gMMPsychicD, false)
		end
	
	--Multiplier for how much strength the psychic will inherit frome the player's strongest trainable unit
		local iFactor = .75
		local iProgress = g_tSavedData.GetValue("Player " .. iPlayer .. " Sanctuaries Discovered") or 0
--		if iProgress > 0 then iProgress = iProgress - 1 end
		if iProgress == 8 then 
			--UpdatePromotionToUnitType(iPlayer, gPsychic, gTeleportII, true)
			pUnit:SetHasPromotion(gTeleportII, true)
			iFactor = 1.25
		elseif iProgress >= 5 then 
			pUnit:SetHasPromotion(gTeleportI, true) 
			iFactor = 1.00 
		elseif iProgress >= 3 then
			iFactor = .85
		end
		
		
		iStrength = math.ceil(GetStrongestMilitaryUnitStrength(pPlayer, true, "UNITCOMBAT_MELEE", "UNITCOMBAT_GUN") * iFactor)
		if iStrength > 10 then 
			pUnit:SetBaseCombatStrength(iStrength)
		end
	end
end
Events.SerialEventUnitCreated.Add(EaglePsychicCreated)

function MMPsychicKilled(iOwner, iKilledUnit, iUnitType, x, y, bDelay, iKillerOwner)
	if not g_tSavedData.GetValue("Player " .. iOwner .. " Sanctuaries Discovered") then return end
	if not (g_tSavedData.GetValue("Player " .. iOwner .. " Sanctuaries Discovered") >= 5) then return end
	if iOwner == iKillerOwner or iKillerOwner == -1 then return end
	local pPlayer = Players[iOwner]
	local pUnit = pPlayer:GetUnitByID(iKilledUnit)
	if pUnit:IsHasPromotion(gPSIShield) then DamageReflect(pUnit, iOwner) end
end
GameEvents.UnitPrekill.Add(MMPsychicKilled)	
--------------------
--InitEagle
--Initialization function, fires on game load
--------------------
function InitEagle()
	for iPlayer = 0,  GameDefines.MAX_MAJOR_CIVS - 1 do
		local pPlayer = Players[iPlayer]
		local iTeam = pPlayer:GetTeam()
			if not tEagleTeams[iTeam] then
				tEagleTeams[iTeam] = {}
				tEagleTeams[iTeam][iPlayer] = true
			else 
				tEagleTeams[iTeam][iPlayer] = true
			end
			--tMMPsychics[playerID] = {}
			local iNaturalWondersFound = pPlayer:GetHappinessFromNaturalWonders() / gNaturalWonderHappiness
			if iNaturalWondersFound == 0 then
				--Starting in later eras causes the player receive these techs for free
				--Remove all techs, just in case
				local pTeam = Teams[pPlayer:GetID()]
				pTeam:SetHasTech(tNessProgress[1], false)
				pTeam:SetHasTech(tNessProgress[2], false)
				pTeam:SetHasTech(tNessProgress[3], false)
				pTeam:SetHasTech(tNessProgress[4], false)
				pTeam:SetHasTech(tNessProgress[5], false)
				pTeam:SetHasTech(tNessProgress[6], false)
				pTeam:SetHasTech(tNessProgress[7], false)
				pTeam:SetHasTech(tNessProgress[8], false)
			end
		if pPlayer:GetCivilizationType() == gEagleland and not pPlayer:IsHuman() then
			Events.SequenceGameInitComplete.Add(InjectNessUniqueDiplo);
		end
	end
end
InitEagle()
				



				
