
-- Author: Marius
-- DateCreated: 10/7/2018 8:37:09 PM

--------------------------------------------------------------
include "MM_MapSweeper.lua"
include "UniqueDiplomacyUtilsV3.lua"
include "MMPSIFunctions.lua"
gEagleland = GameInfoTypes.CIVILIZATION_MM_EAGLELAND
------------------------------------------
--Drugstore
------------------------------------------
gDrugstore = GameInfoTypes.BUILDING_MM_DRUGSTORE
gDrugstoreBonus = GameInfoTypes.BUILDING_MM_DRUGSTORE_DUMMY
-----------------------------------
--Psychic
-----------------------------------
gMMPsychic = GameInfoTypes.UNIT_MM_PSYCHIC

gMMPsychicD = GameInfoTypes.PROMOTION_MM_PSYCHIC_D
gTeleportI = GameInfoTypes.PROMOTION_MM_PSI_TELEPORT_I
gTeleportII = GameInfoTypes.PROMOTION_MM_PSI_TELEPORT_II
gPSIShield = GameInfoTypes.PROMOTION_MM_PSI_SHIELD

----------
--Bus Stop
----------
gBusStop = GameInfoTypes.BUILDING_MM_BUS_STOP
gEagleGold = GameInfoTypes.BUILDING_MM_1_GOLD
gTradeRoute1 = GameInfoTypes.BUILDING_MM_EAGLE_TRADE_ROUTE
-----------------
--Escargo Express
-----------------
gEscargoExpressMain = GameInfoTypes.UNIT_MM_ESCARGO_MAN
gEscargoExpressCarry = GameInfoTypes.UNIT_MM_ESCARGO_CARRY
gEscargoPickup = GameInfoTypes.IMPROVEMENT_MM_ESCARGO_PICKUP
gEscargoDelivery = GameInfoTypes.IMPROVEMENT_MM_ESCARGO_DELIVERY
-----------------------------
--Variable for holding the amount of happiness from each natural wonders
--Since there is no method I know of for getting the amount of natural wonders on a map,
--We have to do iHappinessPerNW = Player:GetHappinessFromNaturalWonders() / gNaturalWonderHappiness
--To get the amount of natural wonders the player has found
------------------------------
gNaturalWonderHappiness = 1 
for row in DB.Query("SELECT * FROM Defines WHERE Name = 'HAPPINESS_PER_NATURAL_WONDER'") do
	gNaturalWonderHappiness = row.Value
end
dprint(gNaturalWonderHappiness)
--UA Stuff
tNessProgress = {
	[1] = GameInfoTypes.TECH_MM_EARTHBOUND_I,
	[2] = GameInfoTypes.TECH_MM_EARTHBOUND_II,
	[3] = GameInfoTypes.TECH_MM_EARTHBOUND_III,
	[4] = GameInfoTypes.TECH_MM_EARTHBOUND_IV,
	[5] = GameInfoTypes.TECH_MM_EARTHBOUND_V,
	[6] = GameInfoTypes.TECH_MM_EARTHBOUND_VI,
	[7] = GameInfoTypes.TECH_MM_EARTHBOUND_VII,
	[8] = GameInfoTypes.TECH_MM_EARTHBOUND_VIII
	}
--Happiness dummy
gNessHappiness = GameInfoTypes.BUILDING_MM_HAPPINESS_5
--Great Person generation dummy
gNessGP = GameInfoTypes.BUILDING_MM_GP_GEN
--For easy checks for whether or not dummy buildings are cities they shouldn't be in
tMMEagleCapitalDummies = {
	[gNessHappiness] = true,
	[gNessGP] = true
	}

--PSI Globals
g_iPKFire = 1
g_iPKFreeze = 2
g_iPKThunder = 3
g_iLifeup = 4
g_iPSIShield = 5
g_iHypnosis = 6
-------------------------------------------------------------------------------------------------------------------------
-- GetStrongestMilitaryUnit
-- Code from BouncyMishca's Hakurei Shrine mod
-------------------------------------------------------------------------------------------------------------------------
function GetStrongestMilitaryUnitStrength(pPlayer, bIgnoreResources, ...)
	local tUnit = {["Combat"] = 0}
	for iKey, sCombatType in pairs(arg) do
		for row in GameInfo.Units("CombatClass = \'" .. sCombatType .. "\'") do
			if pPlayer:CanTrain(row.ID, bIgnoreResources) and row.Combat > tUnit.Combat then
				tUnit = row
			end
		end
	end
	return tUnit.Combat
end
------------------------------------------------------
--GetHappinessFromWorkedNaturalWonders
--Since Player:GetHappinessFromNaturalWonders includes happiness from Natural wonders the player owns (Think Fountain of Youth)
--We need a method to seperate that happiness from happiness via discovering natural wonders
--This function is broad-strokes approach to doing so
------------------------------------------------------

function GetHappinessFromWorkedNaturalWonders(iPlayer)
	iStart = os.clock()
	local iHappiness = 0
	for x = 0, MAP_X - 1, 1 do
		for y = 0, MAP_Y - 1, 1 do
			local pPlot = Map.GetPlot(x, y)
			if pPlot then
				if pPlot:GetOwner() == iPlayer and tHappinessFeatures[pPlot:GetFeatureType()] then
					 iHappiness = iHappiness +  tHappinessFeatures[pPlot:GetFeatureType()]
				end
			end
		end
	end
	return iHappiness
end

--The map has not been checked for Natural Wonders yet... that, or there are none.
--Basically, since Nowhere and Eagleland share the same file, this prevents redundant calls to Sweep 
if gNumNaturalWonders == 0 then Sweep() end
				



-- ============================================================================
-- SubstituteEndNumberForPercentSign
-- ============================================================================
-- From VV - Replaces the number at the end of a text tag to a percentage sign
function SubstituteEndNumberForPercentSign(sString)
--	dprint("String before substitution: "..sString)
	local sReverse = string.reverse(sString)
	local iStop = string.find(sReverse, "_")
	if not iStop then return sString end
	sReverse = string.sub(sReverse, iStop + 1)
	sString = string.reverse(sReverse).."%"
--	dprint("String after substitution: "..sString)
	return sString
end

--------------------------------
--InjectNessUniqueDiplo
--This function is fired on game load, when a Ness AI is detected
--Injects unique dialogue if a human player is playing a recognized leader
--------------------------------
tNessUniqueLeaders = {
	["LEADER_PORKY"] = true,
	["LEADER_MM_PORKY"] = true,
	["LEADER_MM_LUCAS"] = true,
	["LEADER_STARMAN_DX"] = true,
	["LEADER_MM_NINTEN"] = true,
	["LEADER_MARIO"] = true,
	["LEADER_OZAI"] = true,
	["LEADER_FINN"] = true,
	["LEADER_BM_EXULON"] = true
}
function InjectNessUniqueDiplo()
	for iPlayer = 0, GameDefines.MAX_MAJOR_CIVS - 1 do
		local pPlayer = Players[iPlayer]
		if pPlayer:IsEverAlive() then
			local eLeader = GameInfo.Leaders[pPlayer:GetLeaderType()].Type
			if pPlayer:IsHuman() and tNessUniqueLeaders[eLeader] then
				local sLeader = string.sub(eLeader, 7, string.len(eLeader ))
				local tResponses = {}
				--Just to save me the headache of having to duplicate these
				if sLeader == "_MM_PORKY" then sLeader = "_PORKY" end
				if sLeader == "_MM_FINN" then sLeader = "_FINN" end
				print(sLeader)
				--I see many people do ChangeDiplomacyResponse manually, but that seems like too much of a hassle
				--As you'll need to add more code in lua everytime you add a new response you want to replace
				--This method is just 'set it and forget it', I just add my lines in XML and that's that
				--TODO: consider supporting other languages for this Query
				for row in DB.Query("SELECT * FROM Language_en_US WHERE Tag LIKE ('TXT_KEY_LEADER_MM_NESS_TO" .. sLeader .. "%')") do
				--26
					local sTag = SubstituteEndNumberForPercentSign(row.Tag)
					if not tResponses[sTag] then
					
						tResponses[sTag] = true
						--When I do unique dialog, I always follow the same cadence I did for the line I am replacing
						--That is the only way this Query will work
						--What we're doing is we're cutting out the middle section of 'TXT_KEY_LEADER_MM_NESS_TO_LEADERNAME_RESPONSE%'
						--To just be 'TXT_LEADER_MM_NESS_RESPONSE%'
						--That way we can find the orignal ResponseType this line is meant to replace
						--print(string.sub(row.Tag, string.find(row.Tag, "TO") + string.len(sLeader) + 2, string.len(row.Tag) - 2))
						for subrow in DB.Query("SELECT * FROM Diplomacy_Responses WHERE Response LIKE ('TXT_KEY_LEADER_MM_NESS" .. string.sub(sTag, string.find(sTag, "TO") + string.len(sLeader) + 2, string.len(sTag)) .. "')") do
							dprint(subrow.ResponseType)
							ChangeDiplomacyResponse("LEADER_MM_NESS", subrow.ResponseType, sTag)
						end
					end
				end
				--Remove this leader from NessUniqueLeaders to help prevent redundant calls to ChangeDiplomacyResponse
				--tNessUniqueLeaders[eLeader] = nil	
				--Actually, we only want to run this once after we found our leader
				return
			end
		end
	end
end

