-- MMPSIFunctions
-- Author: Marius
-- DateCreated: 5/21/2019 8:51:14 AM
--------------------------------------------------------------
include "PlotIterators.lua"
g_tSavedData = Modding.OpenSaveData()



-------------------------
--		UTILITIES
-------------------------
bMMDebug = false
function dprint(...)
	if (bMMDebug) then print(...) end
end


function MMGetUnitType(pUnit)
	return GameInfo.Units[pUnit:GetUnitType()].Type
end

tValidPSIusers = {} --Units that can use PSI
local tPSITechniques = {} --All defined PSI Tech
local tPSIGroups = {} --All defined PSI Groups
--Stores enemies by index
tMMNessEnemies = {}
--Stores enemies by their ID
tMMEnemies = {}
--Stores friendly units by index
tMMNessFriends = {}
--Stores friendly units by their ID
tMMFriends = {}
--Stores Hypnotized Enemies
--Unit:Kill() does not seem to immediately register the unit as dead
--Meaning that other Psychics could potentially target a unit that no longer exists
--And even hypnotize it again, giving the player a second copy of it!

tMMHypnotizedEnemies = {}
---------------------------------------------------
--GetNumNearbyEnemies
--Iterates through all plots iRange away from x, y
--Count up all units in iRange that is hostile to iPlayer, 
--And add each hostile unit to tMMNessEnemies and tMMEnemies
--NEW: Also gets nearby friendly units
---------------------------------------------------
function GetNumNearbyEnemies(x, y, iPlayer, iRange)
	local iEnemies = 0
	local iFriends = 0
	local pTeam = Teams[Players[iPlayer]:GetTeam()]
	for pPlot in PlotAreaSweepIterator(Map.GetPlot(x, y), iRange, SECTOR_NORTH, DIRECTION_CLOCKWISE, DIRECTION_OUTWARDS, false) do
		for i = 0, (pPlot:GetNumUnits() - 1) do
			local pUnit = pPlot:GetUnit(i)
			if pUnit:IsCombatUnit() and tMMHypnotizedEnemies[pUnit:GetID()] == nil then
				if pTeam:IsAtWar(pUnit:GetOwner()) then
					tMMNessEnemies[iEnemies] = {}
					tMMNessEnemies[iEnemies].ID = pUnit:GetID()
					tMMNessEnemies[iEnemies].Player = pUnit:GetOwner()
					tMMEnemies[pUnit:GetID()] = true
					iEnemies = iEnemies + 1 
				elseif pUnit:IsCombatUnit() and tMMHypnotizedEnemies[pUnit:GetID()] == nil and pUnit:GetOwner() == iPlayer then
					tMMNessFriends[iFriends] = {}
					tMMNessFriends[iFriends].ID = pUnit:GetID()
					tMMNessFriends[iFriends].Player = iPlayer
					tMMFriends[pUnit:GetID()] = true
					iFriends = iFriends + 1
				end
			end
		end
	end
	return iEnemies, iFriends
end

--------------------------------------------------
--AdjustWeights
--Takes a PSIGroup and an already defined amount of nearby friends and enemies
--	In other words, this function assumes GetNumNearbyEnemies has already been called and iEnemies and iFriends has been determined by that
--Checks if PSI can be used depending on if there are nearby friends or enemies or that the PSI will allow such usage.
--------------------------------------------------
function AdjustWeights(sGroup, iEnemies, iFriends)
	dprint("Adjusting weights based on proximity of friends and foes")
	local bEnemies = iEnemies > 0 
	local bFriends = iFriends > 0	
	--if bEnemies then
	local tGroup = tPSIGroups[sGroup]
	for i = 0, #tPSIGroups[sGroup], 1 do
		--If the move cannot be used on yourself, there are no enemies or the move cannot be used on enemies, AND there are no friends or the move cannot be used on friends
		if not ((tPSITechniques[tGroup[i].PSI].Self) or (bEnemies and tPSITechniques[tGroup[i].PSI].Enemy) or (bFriends and tPSITechniques[tGroup[i].PSI].Friendly)) then
			dprint("Disabling PSI " .. tGroup[i].PSI)
			tPSIGroups[sGroup][i].Weight = 0
		end
	end
end

-------------------------------------
--CalculateTotalWeight
--Calculates the current defined weights at the passed PSIGroup
-------------------------------------		
function CalculateTotalWeight(sGroup)
	dprint("Calculating total weights")
	local iWeight = 0
	for i = 0, #tPSIGroups[sGroup], 1 do
		iWeight = iWeight + tPSIGroups[sGroup][i].Weight
	end
return iWeight end


-----------------------------
--ResetWeights
--Resets the weights at the passed PSIGroup to what they were intially defined as
-----------------------------
function ResetWeights(sGroup)
	dprint("Resetting weights")
	for i = 0, #tPSIGroups[sGroup], 1 do
		tPSIGroups[sGroup][i].Weight = tPSIGroups[sGroup][i].WEIGHT
	end
end
-----------------------------------------
--ChoosePSI
--Taking the total weight of passed PSIGroup
--AND an already determined random number (iRoll)
--Figure out which PSI in the group that is
-----------------------------------------
function ChoosePSI(iChain, iRoll, sGroup)
	dprint(sGroup .. ": Rolled a " .. iRoll)
	local tGroup = tPSIGroups[sGroup]
	local iPSI = nil
	local iTotal = 0
	local iCurrent = 0
	for i = 0, #tGroup, 1 do
		iTotal = iTotal + tGroup[i].Weight
		dprint("iCurrent " .. iCurrent)
		dprint("iTotal " .. iTotal)
		if iRoll > iCurrent and iRoll <= iTotal and tGroup[i].Weight > 0 then
			iPSI = i
		end
		iCurrent = iTotal
		
	end
	return iPSI
end

------------------------------
--GetTargetUnit
--Assumes tMMNessEnemies has been filled 
--Gets a random unit from that table
------------------------------
function GetTargetUnit(iUnits)
	local iUnitRoll = Game.Rand(iUnits, "Ness Roll")
	local iTargetUnit = nil
	local pTargetPlayer = nil
	local pTargetUnit = nil
	if tMMNessEnemies[iUnitRoll] then
		iTargetUnit = tMMNessEnemies[iUnitRoll].ID
		pTargetPlayer = Players[tMMNessEnemies[iUnitRoll].Player]
		pTargetUnit = pTargetPlayer:GetUnitByID(iTargetUnit) 
	end
	return pTargetUnit
end

-----------------------------
--DamageReflect
--Used to damage nearby units when a PSI shield is expelled
-----------------------------
function DamageReflect(pUnit)
	dprint("This Psychic had a PSI Shield, reflecting damage")
	dprint("Health was " .. g_tSavedData.GetValue("Unit" .. pUnit:GetID() .. "_CurrentHealth"))
	dprint("Unit has taken " .. pUnit:GetDamage() .. " damage")
	local iPlayer = pUnit:GetOwner()
	local iDamage = g_tSavedData.GetValue("Unit" .. pUnit:GetID() .. "_CurrentHealth") - (100 - pUnit:GetDamage())
	if iDamage > 0 then
		local iUnits = GetNumNearbyEnemies(pUnit:GetX(), pUnit:GetY(), iPlayer, 2)
		if iUnits > 0 then
			iDamage = math.floor(iDamage / iUnits)
			dprint(iDamage .. "damage distributed across " .. iUnits .. " enemies")
			for k, v in pairs(tMMNessEnemies) do
				local pTargetUnit = Players[v.Player]:GetUnitByID(v.ID)
				pTargetUnit:ChangeDamage(iDamage)
				Players[pTargetUnit:GetOwner()]:AddNotification(NotificationTypes.NOTIFICATION_GREAT_PERSON_ACTIVE_PLAYER, "An enemy " .. pUnit:GetUnitType() .. " reflected its shield, dealing " .. iDamage .. " damage to your " .. pTargetUnit:GetName() .. " !", "Shield Reflect!", pTargetUnit:GetX(), pTargetUnit:GetY(), pTargetUnit:GetUnitType())
			end
			g_tSavedData.SetValue("Unit" .. pUnit:GetID() .. "_CurrentHealth", nil)
			tMMEnemies = {}
			tMMNessEnemies = {}
			Players[iPlayer]:AddNotification(NotificationTypes.NOTIFICATION_GREAT_PERSON_ACTIVE_PLAYER, "Your " .. pUnit:GetUnitType() .. " reflected their PSI Shield, dealing " .. iDamage .. " damage to " .. iUnits .. " enemies!", "Shield Reflect!", pUnit:GetX(), pUnit:GetY(), gMMPsychic)
		end
	end
end

--------------------------------------------
-- Data Initialization
--------------------------------------------

--Fill our lua tables with data
for row in DB.Query("SELECT * FROM MMUnit_PSIGroup") do
	tValidPSIusers[row.UnitType] = row.PSIGroup
end

for row in DB.Query("SELECT * FROM MMPSI") do
	tPSITechniques[row.Type] = {}
	tPSITechniques[row.Type].Self = row.Self == 1 --SQLite does not have a boolean datatype, but this will convert as intended
	tPSITechniques[row.Type].Enemy = row.Enemy == 1
	tPSITechniques[row.Type].Friendly = row.Friendly == 1
end


for k, v in pairs(tValidPSIusers) do
	tPSIGroups[v] = {}
	local i = 0
	for row in DB.Query("SELECT * FROM MMPSIGroup WHERE Type = '" .. v .. "'") do
		tPSIGroups[v][i] = {}
		tPSIGroups[v][i].PSI = row.PSI
		tPSIGroups[v][i].Weight = row.Weight --This is mutable
		tPSIGroups[v][i].WEIGHT = row.Weight --And this is not
		i = i + 1
	end
end


--===================================================
--Define what happens when each PSI technique is used
--===================================================
tPSITechniques["PSI_PK_FIRE"].Action = (function(pUnit, pTargetUnit, iDamage, eData)
	print("Using PK Fire!")
	local pPlayer = Players[pUnit:GetOwner()]
	local pTargetPlayer = Players[pTargetUnit:GetOwner()]
	local pTeam = Teams[pPlayer:GetTeam()]
	if not iDamage then iDamage = 15 end
	--Iterate through plots adjacent to the unit, damage all enemies
	for pPlot in PlotAreaSweepIterator(Map.GetPlot(pTargetUnit:GetX(), pTargetUnit:GetY()), 1, SECTOR_NORTH, DIRECTION_CLOCKWISE, DIRECTION_OUTWARDS, CENTRE_INCLUDE) do
		for i = 0, (pPlot:GetNumUnits() - 1) do
			local pCurrentUnit = pPlot:GetUnit(i)
			if pTeam:IsAtWar(pCurrentUnit:GetOwner()) then
				pCurrentUnit:ChangeDamage(iDamage)
			end
		end
	end
	pPlayer:AddNotification(NotificationTypes.NOTIFICATION_GREAT_PERSON_ACTIVE_PLAYER, "Your " .. pUnit:GetName() .. " tried PK Fire, damaging an enemy " .. pTargetUnit:GetName() .. " and all enemies near it!", "PK Fire!", pTargetUnit:GetX(), pTargetUnit:GetY(), pTargetUnit:GetUnitType())
	pTargetPlayer:AddNotification(NotificationTypes.NOTIFICATION_GREAT_PERSON_ACTIVE_PLAYER, "An enemy " .. pUnit:GetName() .. " tried PK Fire on your " .. pTargetUnit:GetName() .. "!", "PK Fire!", pUnit:GetX(), pUnit:GetY(), pUnit:GetUnitType())
end)

tPSITechniques["PSI_PK_FREEZE"].Action = (function(pUnit, pTargetUnit, iDamage, eData)
	print("Using PK Freeze!")
	local pPlayer = Players[pUnit:GetOwner()]
	local pTargetPlayer = Players[pTargetUnit:GetOwner()]
	if not iDamage then iDamage = 0 end
	pTargetUnit:FinishMoves()
	pTargetUnit:ChangeDamage(iDamage)
	pPlayer:AddNotification(NotificationTypes.NOTIFICATION_GREAT_PERSON_ACTIVE_PLAYER, "Your " .. pUnit:GetName() .. " tried PK Freeze, solidifying an enemy " .. pTargetUnit:GetName() .. "!", "PK Freeze!", pTargetUnit:GetX(), pTargetUnit:GetY(), pTargetUnit:GetUnitType())
	pTargetPlayer:AddNotification(NotificationTypes.NOTIFICATION_GREAT_PERSON_ACTIVE_PLAYER, "An enemy " .. pUnit:GetName() .. " tried PK Freeze on your " .. pTargetUnit:GetName() .. "! Its body solidified and is unable to move!", "PK Freeze!", pUnit:GetX(), pUnit:GetY(), pUnit:GetUnitType())
end)

tPSITechniques["PSI_PK_THUNDER"].Action = (function(pUnit, pTargetUnit, iDamage, iTargets)
	print("Using PK Thunder!")
	local pPlayer = Players[pUnit:GetOwner()]
	local pTargetPlayer = Players[pTargetUnit:GetOwner()]
	if not iDamage then iDamage = 20 end
	if not iTargets then iTargets = Game.Rand(2, "Ness Roll") + 1 end							  
	pTargetUnit:SetHasPromotion(GameInfoTypes.PROMOTION_NO_DEFENSIVE_BONUSES, true)
	pTargetUnit:ChangeDamage(iDamage)
	dprint("Targeting " .. pTargetUnit:GetID())
	pPlayer:AddNotification(NotificationTypes.NOTIFICATION_GREAT_PERSON_ACTIVE_PLAYER, "Your " .. pUnit:GetName() .. " tried PK Thunder on " .. iTargets .. " enemies! They have permanently lost defensive terrain bonuses!", "PK Thunder!", pTargetUnit:GetX(), pTargetUnit:GetY(), pTargetUnit:GetUnitType())
	pTargetPlayer:AddNotification(NotificationTypes.NOTIFICATION_GREAT_PERSON_ACTIVE_PLAYER, "An enemy " .. pUnit:GetName() .. " tried PK Thunder on " .. iTargets .. " of your units! They have permanently lost defensive terrain bonuses!", "PK Thunder!", pUnit:GetX(), pUnit:GetY(), pUnit:GetUnitType())
	if iTargets > 1 then
		local iNumEnemies = GetNumNearbyEnemies(pUnit:GetX(), pUnit:GetY(), pPlayer:GetID(), 2)
		while iTargets > 1 do
			iUnitRoll = Game.Rand(iNumEnemies, "Ness Roll")
		--	while(tMMNessEnemies[iUnitRoll].ID == pTargetUnit:GetID()) do
		--		iUnitRoll = Game.Rand(iNumEnemies, "Ness Roll")
		--	end
			local pCurrentUnit = Players[tMMNessEnemies[iUnitRoll].Player]:GetUnitByID(tMMNessEnemies[iUnitRoll].ID)
			dprint("Targeting " .. pCurrentUnit:GetID())
			pCurrentUnit:SetHasPromotion(GameInfoTypes.PROMOTION_NO_DEFENSIVE_BONUSES, true)
			pCurrentUnit:ChangeDamage(iDamage)
			iTargets = iTargets - 1
		end
	end
end)

tPSITechniques["PSI_LIFEUP"].Action = (function(pUnit, pTargetUnit, iHealth, eData)
	local pPlayer = Players[pUnit:GetOwner()]
	if pUnit:GetDamage() == 0 then return end
	if not iHealth then iHealth = -20 end
	dprint("Using Lifeup!")
	pPlayer:AddNotification(NotificationTypes.NOTIFICATION_GREAT_PERSON_ACTIVE_PLAYER, "Your " .. pUnit:GetName() .. " tried Lifeup, recovering " .. -iHealth .. " HP!", "Lifeup!", pUnit:GetX(), pUnit:GetY(), pUnit:GetUnitType())
	pUnit:ChangeDamage(iHealth)
end)

tPSITechniques["PSI_PSI_SHIELD"].Action = (function(pUnit, eData, bReflect, eData2)
	local pPlayer = Players[pUnit:GetOwner()]
	if not tMMNessEnemies[0] then return end --No enemies, no reason to use
	dprint("Using PSI Shield!")
	pUnit:SetHasPromotion(gPSIShield, true)
	if bReflect then
		g_tSavedData.SetValue("Unit" .. pUnit:GetID() .. "_CurrentHealth", 100 - pUnit:GetDamage())
	end
	pPlayer:AddNotification(NotificationTypes.NOTIFICATION_GREAT_PERSON_ACTIVE_PLAYER, "Your " .. pUnit:GetName() .. " tried PSI Shield, increasing their defensive strength by 50%!", "PSI Shield!", pUnit:GetX(), pUnit:GetY(), pUnit:GetUnitType())
end) 

tPSITechniques["PSI_HYPNOSIS"].Action = (function(pUnit, pTargetUnit, eData, eData2)
	dprint("Using Hypnosis!")
	local pPlayer = Players[pUnit:GetOwner()]
	local pTargetPlayer = Players[pTargetUnit:GetOwner()]
	pPlayer:AddNotification(NotificationTypes.NOTIFICATION_GREAT_PERSON_ACTIVE_PLAYER, "Your " .. pUnit:GetName() .. " tried Hypnosis on an enemy " .. pTargetUnit:GetName() .. ", bringing it under your control!", "Hypnosis!", pTargetUnit:GetX(), pTargetUnit:GetY(), pTargetUnit:GetUnitType())
	pTargetPlayer:AddNotification(NotificationTypes.NOTIFICATION_GREAT_PERSON_ACTIVE_PLAYER, "An enemy " .. pUnit:GetName() .. " tried Hypnosis on your" .. pTargetUnit:GetName() .. ", bringing it under their control!", "Hypnosis!", pTargetUnit:GetX(), pTargetUnit:GetY(), pTargetUnit:GetUnitType())
	--The game will CTD if the target unit is in a city
	if pTargetUnit:GetPlot():GetPlotCity() then
		--In BNW, JumpToNearestValidPlot will always move the unit 
		--even if the plot it's already on is already 'valid'
		--This is not the case with VP. Force it so.
		local pDummyUnit = pTargetPlayer:InitUnit(pTargetUnit:GetUnitType(), pTargetUnit:GetX(), pTargetUnit:GetY(), UNITAI_ATTACK)
		pTargetUnit:JumpToNearestValidPlot()
		pDummyUnit:Kill()
	end
	tMMHypnotizedEnemies[pTargetUnit:GetID()] = true
	local pNewUnit = pPlayer:InitUnit(pTargetUnit:GetUnitType(), pTargetUnit:GetX(), pTargetUnit:GetY(), UNITAI_ATTACK)
	--pTargetUnit:Convert(pNewUnit)
	pNewUnit:Convert(pTargetUnit)
end)

tPSITechniques["PSI_PK_BEAM"].Action = (function(pUnit, pTargetUnit, iDamage, eData) 
	dprint("Using PK Beam!")
	local pPlayer = Players[pUnit:GetOwner()]
	local pTargetPlayer = Players[pTargetUnit:GetOwner()]
	if not iDamage then iDamage = 15 end
	pTargetUnit:ChangeDamage(iDamage)
	pPlayer:AddNotification(NotificationTypes.NOTIFICATION_GREAT_PERSON_ACTIVE_PLAYER, "Your " .. pUnit:GetName() .. " tried PK Beam on an enemy " .. pTargetUnit:GetName() .. ", dealing " .. iDamage .. " damage!", "PK Beam!", pTargetUnit:GetX(), pTargetUnit:GetY(), pTargetUnit:GetUnitType())
	pTargetPlayer:AddNotification(NotificationTypes.NOTIFICATION_GREAT_PERSON_ACTIVE_PLAYER, "An enemy " .. pUnit:GetName() .. " tried PK Beam on your" .. pTargetUnit:GetName() .. ", dealing " .. iDamage .. "damage!", "PK Beam!", pTargetUnit:GetX(), pTargetUnit:GetY(), pTargetUnit:GetUnitType())
end)

tPSITechniques["PSI_PK_STARSTORM"].Action = (function(pUnit, eData, iDamage, eData2)
	dprint("Using PK Starstorm")
	local pPlayer = Players[pUnit:GetOwner()]
	tTargets = {}
	for k, v in pairs(tMMNessEnemies) do
		iTargetUnit = tMMNessEnemies[k].ID
		pTargetPlayer = Players[tMMNessEnemies[k].Player]
		if not tTargets[pTargetPlayer:GetID()] then tTargets[pTargetPlayer:GetID()] = pTargetPlayer end
		pTargetUnit = pTargetPlayer:GetUnitByID(iTargetUnit) 
		if not iDamage then iDamage = Game.Rand(31, "Ness Roll") + 20 end --Will range from 20 to 50
		dprint(iDamage .. " damage")
		pTargetUnit:ChangeDamage(iDamage)
	end
	pPlayer:AddNotification(NotificationTypes.NOTIFICATION_GREAT_PERSON_ACTIVE_PLAYER, "Your " .. pUnit:GetName() .. " unleashed PK Starstorm, damaging all nearby enemies!", "PK Starstorm!", pUnit:GetX(), pUnit:GetY(), pUnit:GetUnitType())
	for k, v in pairs(tTargets) do
		v:AddNotification(NotificationTypes.NOTIFICATION_GREAT_PERSON_ACTIVE_PLAYER, "An enemy " .. pUnit:GetName() .. " unleashed PK Starstorm, damaging all nearby enemies!", "PK Starstorm!", pUnit:GetX(), pUnit:GetY(), pUnit:GetUnitType())
	end
end)

--====================================================================
--Define functions for each PSIGroup that determines how PSI is chosen
--====================================================================
tPSIGroups["MM_EAGLE"].Action = (function(pPlayer, pUnit, sGroup)
	--Keep in mind, Eagleland's PSI is supposed to change quite dynamically, based on UA progress
	local iPlayer = pPlayer:GetID()
	local iProgress = g_tSavedData.GetValue("Player " .. pPlayer:GetID() .. " Sanctuaries Discovered") or 0
	local iPKFire = 0
	local iPKFreeze = 1
	local iPKThunder = 2
	local iLifeup = 3
	local iPSIShield = 4
	local iHypnosis = 5
	local tGroup = tPSIGroups["MM_EAGLE"]
	--When in debug mode, PSI usage can be forced if the player gives themselves one of the Sanctuary dummy techs
	if bMMDebug then
		local iPSI = nil
		local pTeam = Teams[pPlayer:GetTeam()]
		if pTeam:IsHasTech(tNessProgress[1]) then iPSI = iPKFire --PK Fire
		elseif pTeam:IsHasTech(tNessProgress[2]) then iPSI = iPKFreeze --PK Freeze
		elseif pTeam:IsHasTech(tNessProgress[3]) then iPSI = iPKThunder--PK Thunder
		elseif pTeam:IsHasTech(tNessProgress[4]) then iPSI = iHypnosis	--Hypnosis
		elseif pTeam:IsHasTech(tNessProgress[5]) then iPSI = iPSIShield --PSI Shield
		end
		if iPSI then
			dprint("Forcing PSI usage "  .. iPSI .. " for " .. iUnit)
			local iEnemies, iFriends = GetNumNearbyEnemies(pUnit:GetX(), pUnit:GetY(), iPlayer, 2)
			if iNumEnemies < 1 then dprint("WARNING: No enemies detected!") 
			end
			local iUnitRoll = Game.Rand(iNumEnemies, "Ness Roll")
			local iTargetUnit = nil
			if tMMNessEnemies[iUnitRoll] then
				iTargetUnit = tMMNessEnemies[iUnitRoll].ID
				iTargetPlayer = tMMNessEnemies[iUnitRoll].Player
			end
			tPSITechniques[tGroup[iPSI].PSI].Action(pUnit, Players[iTargetPlayer]:GetUnitByID(iTargetUnit), 20, 20)
			return		
		end
	end
--	if iProgress > 0 then iProgress = iProgress - 1 end
	--The base chance of using PSI is 1/3
	local iChance = 32
	if iProgress == 8 then iChance = 84
	elseif iProgress >= 5 then iChance = 65
	elseif iProgress >= 3 then iChance = 44
	end
    local iRoll = Game.Rand(100, "Ness roll")
	dprint(iRoll .. " must be less than or equal to " .. iChance)
	if iRoll > iChance then return end --This unit will not use PSI this turn
	dprint("MM_EAGLE: Attempting to use PSI!")
	local iEnemies, iFriends = GetNumNearbyEnemies(pUnit:GetX(), pUnit:GetY(), iPlayer, 2)
	--The base chance of using PK Fire is 19/60
	--The base chance of using PK Freeze is 10/60
	--The base chance of using Lifeup is 10/60
	--The base chance of using PSI Shield is 15/60
	--The base chance of using PK Thunder is 6/60
	--The base chance of using Hypnosis is 0/60
	
	tGroup[iPKFire].Weight = 19
	tGroup[iPKFreeze].Weight = 10
	tGroup[iPKThunder].Weight = 6
	tGroup[iLifeup].Weight = 10
	tGroup[iHypnosis].Weight = 0

	local bEnemies = iEnemies > 0
	local bFriends = iFriends > 0
	local bHurt = pUnit:IsHurt()
	AdjustWeights("MM_EAGLE", iEnemies, iFriends)
	--I was on the fence about having healing moves be handled by AdjustWeights, but I think it might be better to have it be done by individual circumstances
	if not bHurt then tPSIGroups["MM_EAGLE"][iLifeup].Weight = 0 end
	if not bEnemies then tPSIGroups["MM_EAGLE"][iPSIShield].Weight = 0 end
	--PSI moves are chained together in a sequence of integers from which one is chosen at random 
	--EXAMPLE: PK Fire will be used if it ranges from 0-2
	--PK Freeze will be used if it ranges from 3-5, etc.
	local iTotalWeight = CalculateTotalWeight("MM_EAGLE")
	if not bEnemies then dprint("No Enemies") end
	if not bHurt then dprint("Full Health") end
	if iTotalWeight <= 0 then return end
	if bEnemies then
		if iProgress == 8 then
			ResetWeights("MM_EAGLE")
			if not bHurt then tGroup[iLifeup].Weight = 0 end
		--Hypnosis used more often
		elseif iProgress == 7 then
			ResetWeights("MM_EAGLE")
			if not bHurt then tGroup[iLifeup].Weight = 0 end
			tGroup[iHypnosis].Weight = 6
		--Hypnosis can be used
		elseif iProgress >= 4 then
			tGroup[iPKThunder].Weight = 15
			if bHurt then tGroup[iLifeup].Weight = 17 end
			tGroup[iPKFreeze].Weight = 19
			tGroup[iPSIShield].Weight = 19
			tGroup[iHypnosis].Weight = 3
		--Shield used more often
		elseif iProgress == 3 then
			tGroup[iPKThunder].Weight = 15
			if bHurt then tGroup[iLifeup].Weight = 17 end
			tGroup[iPKFreeze].Weight = 19
			tGroup[iPSIShield].Weight = 19
		--PK Thunder and Lifeup used more often
		elseif iProgress == 2 then
			tGroup[iPKThunder].Weight = 15
			if bHurt then tGroup[iLifeup].Weight = 17 end
			tGroup[iPKFreeze].Weight = 19
		--PK Freeze used more often
		elseif iProgress == 1 then
			tGroup[iPKFreeze].Weight = 19
		end
		--If there are quite a few enemies nearby, up the chances of PK Fire and PK Thunder
		if iEnemies > 4 then
			tGroup[iPKThunder].Weight = tGroup[iPKThunder].Weight + 7
			tGroup[iPKFire].Weight = tGroup[iPKFire].Weight + 15
		--Vice-versa
		elseif iEnemies < 3 then
			tGroup[iPKFire].Weight = tGroup[iPKFire].Weight - 10
			if tGroup[iPKFire].Weight < 0 then tGroup[iPKFire].Weight = 0 end
		end
	end
	dprint("HP lost: " .. pUnit:GetDamage())
	if pUnit:GetDamage() >= 75 then
		tGroup[iLifeup].Weight = tGroup[iLifeup].Weight + 75
	elseif pUnit:GetDamage() >= 50 then
		tGroup[iLifeup].Weight = tGroup[iLifeup].Weight + 35
	end

	if pUnit:GetPlot():GetOwner() == iPlayer and (pUnit:GetPlot():GetImprovementType() == GameInfoTypes.IMPROVEMENT_FORT or pUnit:GetPlot():GetImprovementType() == GameInfoTypes.IMPROVEMENT_CITADEL) then
		dprint("Unit is stationed on a defensive structure")
		tGroup[iPSIShield].Weight = tGroup[iPSIShield].Weight + 20
	end

	local pTargetUnit = GetTargetUnit(iEnemies)
	
	--If our target is on a fort or citadel, up the chances of PK Thunder
	--That will remove the defensive bonuses it is receiving from the improvement!
	if pTargetUnit then
		local pTargetPlayer = Players[pTargetUnit:GetOwner()]
		if pTargetUnit:GetPlot():GetOwner() == pTargetPlayer:GetID() and (pTargetUnit:GetPlot():GetImprovementType() == GameInfoTypes.IMPROVEMENT_FORT or pTargetUnit:GetPlot():GetImprovementType() == GameInfoTypes.IMPROVEMENT_CITADEL) then
			dprint("Target is on a defensive structure")
			tGroup[iPKThunder].Weight = tGroup[iPKThunder].Weight + 100
		end
	end
	iTotalWeight = CalculateTotalWeight("MM_EAGLE")
	local iRoll =  Game.Rand(iTotalWeight, "Ness Roll") + 1
	local iPSI = ChoosePSI(iTotalWeight, iRoll, "MM_EAGLE")
	dprint("Using " .. iPSI)
	local iDamage = nil
	local eData = nil
	--Now, on to deciding how much damage to do...
	if iPSI == iPKFire then
		iDamage = 10
		if iProgress == 8 then iDamage = 30
		elseif iProgress >= 5 then iDamage = 20
		elseif iProgress >= 3 then iDamage = 15	
		end
	elseif iPSI == iPKFreeze then
		iDamage = 0
		if iProgress == 8 then iDamage = 15
		elseif iProgress >= 5 then iDamage = 5
		end
	elseif iPSI == iPKThunder then
		iDamage = 20
		eData = 1
		if iProgress == 8 then
			eData = 3
			iDamage = 30
		elseif iProgress >= 5 then
			eData = 2
			iDamage = 25
		end
	elseif iPSI == iLifeup then
		iDamage = -10
		if iProgress == 8 then iDamage = -50
		elseif iProgress >= 5 then iDamage = -30
		elseif iProgress >= 3 then iDamage = -20
		end
	elseif iPSI == iPSIShield then
		if iProgress >= 5 then iDamage = true end --Reflect Damage
	end
	--No need to do anything for hypnosis
	tPSITechniques[tGroup[iPSI].PSI].Action(pUnit, pTargetUnit, iDamage, eData)
end)

tPSIGroups["STARMAN"].Action = (function(pPlayer, pUnit, sGroup)
	local iRoll = Game.Rand(100, "Ness roll")
	if iRoll >= 70 then return end -- 70% chance to use PSI
	dprint("STARMAN: Attempting PSI!")
	local iEnemies, iFriends = GetNumNearbyEnemies(pUnit:GetX(), pUnit:GetY(), pPlayer:GetID(), 2)
	local tGroup = tPSIGroups["STARMAN"]
	local iPSIShield = 1
	local iLifeup = 2
	local iStarstorm = 4
	local bEnemies = iEnemies > 0
	local bFriends = iFriends > 0
	local bHurt = pUnit:IsHurt()
	AdjustWeights("STARMAN", iEnemies, iFriends)
	local iTotalWeight = CalculateTotalWeight("STARMAN")
	if not bEnemies then dprint("No Enemies") end
	if not bHurt then dprint("Full Health") end
	if iTotalWeight <= 0 then return end
	dprint("HP lost: " .. pUnit:GetDamage())
	if pUnit:GetDamage() >= 75 then
		tGroup[iLifeup].Weight = tGroup[iLifeup].Weight + 60
	elseif pUnit:GetDamage() >= 50 then
		tGroup[iLifeup].Weight = tGroup[iLifeup].Weight + 30
	end

	if iEnemies > iFriends then	
		tGroup[iStarstorm].Weight = 12
	end
	if pUnit:GetPlot():GetOwner() == iPlayer and (pUnit:GetPlot():GetImprovementType() == GameInfoTypes.IMPROVEMENT_FORT or pUnit:GetPlot():GetImprovementType() == GameInfoTypes.IMPROVEMENT_CITADEL) then
		dprint("Unit is stationed on a defensive structure")
		tGroup[iPSIShield].Weight = tGroup[iPSIShield].Weight + 20
	end
	local pTargetUnit = GetTargetUnit(iEnemies)
	iTotalWeight = CalculateTotalWeight("STARMAN")
	local iRoll =  Game.Rand(iTotalWeight, "Ness Roll") + 1
	local iPSI = ChoosePSI(iTotalWeight, iRoll, "STARMAN")
	dprint("Using " .. iPSI)
	local iDamage = nil
	local eData = nil
	tPSITechniques[tGroup[iPSI].PSI].Action(pUnit, pTargetUnit, iDamage, eData)
end)




function AttemptPSI(iPlayer, iUnit, sPSIGroup)
	local pPlayer = Players[iPlayer]
	local pUnit = pPlayer:GetUnitByID(iUnit)
	tPSIGroups[sPSIGroup].Action(pPlayer, pUnit, sPSIGroup)
	ResetWeights(sPSIGroup)
	tMMEnemies = {}
	tMMNessEnemies = {}
	tMMFriends = {}
	tMMNessFriends = {}
end

tMMPsychics = {}
function MMPsychicCreated(iPlayer, iUnit)
	local pPlayer = Players[iPlayer]
	local pTeam = Teams[pPlayer:GetTeam()]
	local pUnit = pPlayer:GetUnitByID(iUnit)
	if not pUnit then return end
	if tMMPsychics[iPlayer] then
		if tMMPsychics[iPlayer][iUnit] then return end
	end
	--if not(pPlayer:GetCivilizationType() == gEagleland) then return end
	--if pUnit:GetUnitType() == gMMPsychic then
	if tValidPSIusers[MMGetUnitType(pUnit)] then
		if not tMMPsychics[iPlayer] then tMMPsychics[iPlayer] = {} end
		tMMPsychics[iPlayer][iUnit] = true
	end
end
Events.SerialEventUnitCreated.Add(MMPsychicCreated)

function PSIDoTurn(iPlayer)
	if not tMMPsychics[iPlayer] then return end
	local pPlayer = Players[iPlayer]
	for k, v in pairs(tMMPsychics[iPlayer]) do
		--Is the Psychic still alive?
		if pPlayer:GetUnitByID(k) then
			local pUnit = pPlayer:GetUnitByID(k)
			if pUnit then
				--Update combat strength
				--MMPsychicCreated(iPlayer, k)
				--Check if the unit used PSI shield
				if pUnit:IsHasPromotion(gPSIShield) then
					pUnit:SetHasPromotion(gPSIShield, false)
					--If the player is 5/8 in UA progress, the dispelling of the shield will reflect damage
					if g_tSavedData.GetValue("Unit" .. k .. "_CurrentHealth") then
						DamageReflect(pUnit, iPlayer)				
					end
				end
			end
			AttemptPSI(iPlayer, k, tValidPSIusers[MMGetUnitType(pUnit)])
			--Remove this index from the table, the unit here no longer exists
			else tMMPsychics[iPlayer][k] = nil
		end
	end
end
--if not g_tSavedData.GetValue("MMPSIFunctions_Init") == 1 then
--	print("MMPSIFunctions")
	GameEvents.PlayerDoTurn.Add(PSIDoTurn)
--	g_tSavedData.SetValue("MMPSIFunctions_Init", 1)
--else g_tSavedData.SetValue("MMPSI_Functions_Init", 0)
--end
	
