-- MMEagleDecisions
-- Author: Marius
-- DateCreated: 5/28/2019 8:34:41 AM
--------------------------------------------------------------
--include "MMEagleGlobals.lua"
local g_tSavedData = Modding.OpenSaveData()
--=========================
--	Formalize Mu Training
-- Costs Gold, 1 Magistrate
-- Requires 4/8 Melodies
--=========================
local De_MuTraining = {}
HookDecisionCivilizationIcon(De_MuTraining, "CIVILIZATION_MM_EAGLELAND")
De_MuTraining.Name = "TXT_KEY_DECISIONS_MU_TRAINING"
De_MuTraining.Desc = "TXT_KEY_DECISIONS_MU_TRAINING_DESC"
De_MuTraining.CanFunc = (function(pPlayer)
	--This function is used to determine if the decision is enactable or has been enacted
	--It is to return AT LEAST two booleans, but can return three:
		--The first checks if the Decision should appear at all to the player
		--The second checks if the Decision can be enacted
		--The third checks if the Decision has already been enacted

	if not(pPlayer:GetCivilizationType() == GameInfoTypes.CIVILIZATION_MM_EAGLELAND) then return false, false end
	if load(pPlayer, "De_MuTraining") then
		--The decision has already been enacted!
		De_MuTraining.Desc = Locale.ConvertTextKey("TXT_KEY_DECISIONS_MU_TRAINING_ENACTED_DESC")
		return false, false, true
	end
	local iGoldCost = math.floor(300 * iMod) --iMod is an E&D global used to scale costs by gamespeed
	De_MuTraining.Desc = Locale.ConvertTextKey("TXT_KEY_DECISIONS_MU_TRAINING_DESC", iGoldCost)
	if(pPlayer:GetGold() <= iGoldCost or pPlayer:GetNumResourceAvailable(iMagistrate, false) < 1) then return true, false end
	if(g_tSavedData.GetValue("Player " .. pPlayer:GetID() .. " Sanctuaries Discovered") or 0) < 4 then return true, false end
	return true, true
end)



De_MuTraining.DoFunc = (function(pPlayer)
	local iGoldCost = -(math.floor(300 * iMod))
	pPlayer:ChangeNumResourceTotal(iMagistrate, -1)
	pPlayer:ChangeGold(iGoldCost)
	for i = GameDefines.MAX_MAJOR_CIVS - 1, GameDefines.MAX_CIV_PLAYERS - 1, 1 do
		local pMinor = Players[i]
		if pMinor:IsEverAlive() and Teams[pMinor:GetTeam()]:IsHasMet(pPlayer:GetTeam()) then
			pMinor:ChangeMinorCivFriendshipWithMajor(pPlayer:GetID(), 30)
		end
	end
	save(pPlayer, "De_MuTraining", true)
	g_tSavedData.SetValue("Player " .. pPlayer:GetID() .. " De_MuTraining", 8)
end)

De_MuTraining.Monitors = {}
--City States gifting Psychics will work the same way players get them: whenever they train a unit, there's a chance
--for a Psychic to appear
--Some differences, however:
--	Chances are higher, since City States only train units every so often
--	This is done for any combat unit, not just land ones, again since City States don't train units often
--	City States will keep the original trained unit
De_MuTraining.Monitors[GameEvents.CityTrained] = (function(iPlayer, iCity, iUnit)
	local pPlayer = Players[iPlayer]
	if not pPlayer:IsMinorCiv() then return end
	if not pUnit:IsCombatUnit() then return end
	local pAlly = Players[pPlayer:GetAlly()]
	if not load(pAlly, "De_MuTraining") then return end
	print("Mu Training: Minor Civ trained a unit")
	local iProgress = g_tSavedData.GetValue("Player " .. pAlly:GetID() .. " Sanctuaries Discovered") or 0
	if Game.Rand(36, "Ness Roll") <= iProgress then
		local pUnit = pPlayer:GetUnitByID(iUnit)
		local pNewUnit = pAlly:InitUnit(GameInfoTypes.UNIT_MM_PSYCHIC, pAlly:GetCapitalCity():GetX(), pAlly:GetCapitalCity():GetY(), UNITAI_ATTACK)
		iXP = pUnit:GetExperience() + 15
		pNewUnit:ChangeExperience(iXP)
		--pNewUnit:FinishMoves()
		--pUnit:Kill(true, -1)
		pAlly:AddNotification(NotificationTypes.NOTIFICATION_GREAT_PERSON_ACTIVE_PLAYER, "Using the Mu Training program, your ally " .. pPlayer:GetCivilizationDescription() .. " has managed to train a Psychic!", "A Psychic appears", pNewUnit:GetX(), pNewUnit:GetY(), GameInfoTypes.UNIT_MM_PSYCHIC)
	end
end)

Decisions_AddCivilisationSpecific(GameInfoTypes.CIVILIZATION_MM_EAGLELAND, "De_MuTraining", De_MuTraining)

--=====================================
--Patronize the Sky Runner
--Requires, Flight, Gold, 2 Magistrates
--=====================================
gAirportGold = GameInfoTypes.BUILDING_MM_AIRPORT_GOLD
gAirportMod = GameInfoTypes.BUILDING_MM_AIRPORT_GOLD_MOD
De_SkyRunner = {}
HookDecisionCivilizationIcon(De_SkyRunner, "CIVILIZATION_MM_EAGLELAND")
De_SkyRunner.Name = "TXT_KEY_DECISIONS_SKY_RUNNER"
De_SkyRunner.Desc = "TXT_KEY_DECISIONS_SKY_RUNNER"
De_SkyRunner.CanFunc = (function(pPlayer)
	if not(pPlayer:GetCivilizationType() == GameInfoTypes.CIVILIZATION_MM_EAGLELAND) then return false, false end
	if load(pPlayer, "De_SkyRunner") then
		--The decision has already been enacted!
		De_SkyRunner.Desc = Locale.ConvertTextKey("TXT_KEY_DECISIONS_SKY_RUNNER_ENACTED_DESC")
		return false, false, true
	end
	local iGoldCost = math.floor(500 * iMod) --iMod is an E&D global used to scale costs by gamespeed
	De_SkyRunner.Desc = Locale.ConvertTextKey("TXT_KEY_DECISIONS_SKY_RUNNER_DESC", iGoldCost)
	if(pPlayer:GetGold() <= iGoldCost or pPlayer:GetNumResourceAvailable(iMagistrate, false) < 2) then return true, false end
	if not Teams[pPlayer:GetTeam()]:IsHasTech(GameInfoTypes.TECH_FLIGHT) then return true, false end
	return true, true
end)


De_SkyRunner.DoFunc = (function(pPlayer)
	local iGoldCost = -(math.floor(500 * iMod))
	pPlayer:ChangeNumResourceTotal(iMagistrate, -2)
	pPlayer:ChangeGold(iGoldCost)
	local pCapital = pPlayer:GetCapitalCity()
	pCapital:SetNumRealBuilding(GameInfoTypes.BUILDING_AIRPORT, 1)
	pCapital:SetNumRealBuilding(gAirportGold, 1)
	pCapital:SetNumRealBuilding(gAirportMod, 1)
	save(pPlayer, "De_SkyRunner", true)
end)

De_SkyRunner.Monitors = {}
De_SkyRunner.Monitors[GameEvents.PlayerDoTurn] = (function(iPlayer)
	local pPlayer = Players[iPlayer]
	if not load(pPlayer, "De_SkyRunner") then return end
	local iCities = 0
	for pCity in pPlayer:Cities() do
		if pCity:IsHasBuilding(GameInfoTypes.BUILDING_AIRPORT) then
			iCities = iCities + 1
		end
		if pCity:IsHasBuilding(gAirportGold) and not(pCity:IsCapital()) then
			pCity:SetNumRealBuilding(gAirportGold, 0)
		end
	end
	if iCities > 10 then iCities = 10 end
	pPlayer:GetCapitalCity():SetNumRealBuilding(gAirportMod, iCities)
	pPlayer:GetCapitalCity():SetNumRealBuilding(gAirportGold, 1)
end)
Decisions_AddCivilisationSpecific(GameInfoTypes.CIVILIZATION_MM_EAGLELAND, "De_SkyRunner", De_SkyRunner)





	


