
CREATE TABLE IF NOT EXISTS 
    Leader_SharedTraits (
    LeaderType          text    REFERENCES Leaders(Type)        default null,
    TraitOne            text    REFERENCES Policies(Type)       default null,
    TraitTwo            text    REFERENCES Policies(Type)       default null);


CREATE TABLE IF NOT EXISTS COMMUNITY (
Type text,
Value variant default 0
);
--We need the Community table to hold some data in it
--Or else EXISTS statements will automatically return false without actually checking the query, even if other tables included do have relevant data
--See 3UC/4UC for an example scenario of this
INSERT INTO COMMUNITY
	(Type, Value)
VALUES
	('MM_EAGLELAND', 1);


--====================
--Civ IV Leader Traits
--====================
INSERT OR REPLACE INTO Leader_SharedTraits
            (LeaderType,					TraitOne,					TraitTwo)
VALUES      ('LEADER_MM_NESS',		'POLICY_CHARISMATIC_X',		'POLICY_FINANCIAL_X');

UPDATE Leaders 
SET Description = (CASE WHEN EXISTS(SELECT Type FROM Policies WHERE Type = 'POLICY_PHILOSOPHICAL_X' )
	THEN 'Ness [ICON_HAPPINESS_1][ICON_TRADE]'
	ELSE 'TXT_KEY_LEADER_MM_NESS' END) 
WHERE Type = 'LEADER_MM_NESS';

--==========
--Map Labels
--==========
CREATE TABLE IF NOT EXISTS ML_CivCultures (ID INTEGER PRIMARY KEY AUTOINCREMENT, CivType TEXT, CultureType TEXT, CultureEra TEXT DEFAULT 'ANY');
INSERT INTO ML_CivCultures
            (CivType,                   CultureType,        CultureEra)
VALUES      ('CIVILIZATION_MM_EAGLELAND',		'US_NORTHEAST',	'ANY');

--=======
--3UC/4UC
--=======
INSERT INTO Civilization_UnitClassOverrides
	(CivilizationType, UnitClassType, UnitType)
VALUES
	('CIVILIZATION_MM_EAGLELAND', 'UNITCLASS_MM_ESCARGO_MAN', 
		(CASE WHEN EXISTS(SELECT * FROM Buildings B1, COMMUNITY C1 WHERE (B1.Type = 'BUILDING_3UC_STEEL_MILL') OR (B1.Type = 'BUILDING_AMERICA_RANCH' AND C1.Type = 'COMMUNITY_CORE_BALANCE_BUILDINGS' AND C1.Value = 1))
		THEN 'UNIT_MM_ESCARGO_MAN'
		ELSE NULL END));

INSERT INTO Civilization_BuildingClassOverrides
	(CivilizationType, BuildingClassType, BuildingType)
VALUES
	('CIVILIZATION_MM_EAGLELAND', 'BUILDINGCLASS_3UC_DIVINEBLESSING', NULL),
	('CIVILIZATION_MM_EAGLELAND', 'BUILDINGCLASS_CARAVANSARY',
		(CASE WHEN EXISTS(SELECT * FROM Buildings B1, COMMUNITY C1 WHERE (B1.Type = 'BUILDING_3UC_RANCH') OR (B1.Type = 'BUILDING_AMERICA_RANCH' AND C1.Type = 'COMMUNITY_CORE_BALANCE_BUILDINGS' AND C1.Value = 1 ))
		THEN 'BUILDING_MM_BUS_STOP'
		ELSE 'BUILDING_CARAVANSARY' END));

CREATE TRIGGER VP_MMEagleUniqueComponents
AFTER INSERT ON Buildings
WHEN NEW.Type = 'BUILDING_AMERICA_RANCH' AND EXISTS(SELECT * FROM COMMUNITY WHERE Type = 'COMMUNITY_CORE_BALANCE_UNITS' AND Value = 1)
BEGIN
	INSERT INTO Civilization_UnitClassOverrides
		(CivilizationType,				UnitClassType,				UnitType)
	VALUES
		('CIVILIZATION_MM_EAGLELAND',	'UNITCLASS_MM_ESCARGO_MAN','UNIT_MM_ESCARGO');

	INSERT INTO Civilization_BuildingClassOverrides
		(CivilizationType,				BuildingClassType,				BuildingType)
	VALUES
		('CIVILIZATION_MM_EAGLELAND',	'BUILDINGCLASS_CARAVANSARY',	'BUILDING_MM_BUS_STOP');
		
END;

--==========
--Vox Populi
--==========

UPDATE Technologies
	SET Quote = 'TXT_KEY_TECH_MM_EARTHBOUND_I_VP_QUOTE'
	WHERE Type = 'TECH_MM_EARTHBOUND_I' AND EXISTS (SELECT * FROM COMMUNITY WHERE Type = 'COMMUNITY_CORE_BALANCE' AND Value = 1 );

UPDATE Technologies
	SET Quote = 'TXT_KEY_TECH_MM_EARTHBOUND_II_VP_QUOTE'
	WHERE Type = 'TECH_MM_EARTHBOUND_II' AND EXISTS (SELECT * FROM COMMUNITY WHERE Type = 'COMMUNITY_CORE_BALANCE' AND Value = 1 );

UPDATE Technologies
	SET Quote = 'TXT_KEY_TECH_MM_EARTHBOUND_III_VP_QUOTE'
	WHERE Type = 'TECH_MM_EARTHBOUND_III' AND EXISTS (SELECT * FROM COMMUNITY WHERE Type = 'COMMUNITY_CORE_BALANCE' AND Value = 1 );

UPDATE Technologies
	SET Quote = 'TXT_KEY_TECH_MM_EARTHBOUND_IV_VP_QUOTE'
	WHERE Type = 'TECH_MM_EARTHBOUND_IV' AND EXISTS (SELECT * FROM COMMUNITY WHERE Type = 'COMMUNITY_CORE_BALANCE' AND Value = 1 );

UPDATE Technologies
	SET Quote = 'TXT_KEY_TECH_MM_EARTHBOUND_V_VP_QUOTE'
	WHERE Type = 'TECH_MM_EARTHBOUND_V' AND EXISTS (SELECT * FROM COMMUNITY WHERE Type = 'COMMUNITY_CORE_BALANCE' AND Value = 1 );

UPDATE Technologies
	SET Quote = 'TXT_KEY_TECH_MM_EARTHBOUND_VI_VP_QUOTE'
	WHERE Type = 'TECH_MM_EARTHBOUND_VI' AND EXISTS (SELECT * FROM COMMUNITY WHERE Type = 'COMMUNITY_CORE_BALANCE' AND Value = 1 );

UPDATE Technologies
	SET Quote = 'TXT_KEY_TECH_MM_EARTHBOUND_VII_VP_QUOTE'
	WHERE Type = 'TECH_MM_EARTHBOUND_VII' AND EXISTS (SELECT * FROM COMMUNITY WHERE Type = 'COMMUNITY_CORE_BALANCE' AND Value = 1 );

UPDATE Technologies
	SET Quote = 'TXT_KEY_TECH_MM_EARTHBOUND_VIII_VP_QUOTE'
	WHERE Type = 'TECH_MM_EARTHBOUND_VIII' AND EXISTS (SELECT * FROM COMMUNITY WHERE Type = 'COMMUNITY_CORE_BALANCE' AND Value = 1 );

UPDATE Buildings
	SET PrereqTech = 'TECH_HORSEBACK_RIDING'
	WHERE Type = 'BUILDING_MM_DRUGSTORE' AND EXISTS (SELECT * FROM COMMUNITY WHERE Type = 'COMMUNITY_CORE_BALANCE_BUILDINGS' AND Value = 1 );

UPDATE Buildings
	SET Help = 'TXT_KEY_BUILDING_VP_MM_DRUGSTORE_HELP'
	WHERE Type = 'BUILDING_MM_DRUGSTORE' AND EXISTS (SELECT * FROM COMMUNITY WHERE Type = 'COMMUNITY_CORE_BALANCE_BUILDINGS' AND Value = 1 );

UPDATE Buildings
	SET PrereqTech = 'TECH_CURRENCY'
	WHERE Type = 'BUILDING_MM_BUS_STOP' AND EXISTS (SELECT * FROM COMMUNITY WHERE Type = 'COMMUNITY_CORE_BALANCE_BUILDINGS' AND Value = 1 );

UPDATE Building_YieldChanges
	SET Yield = 3
	WHERE YieldType = 'YIELD_GOLD' AND BuildingType = 'BUILDING_MM_DRUGSTORE' AND EXISTS (SELECT * FROM COMMUNITY WHERE Type = 'COMMUNITY_CORE_BALANCE_BUILDINGS' AND Value = 1 );

UPDATE Building_YieldChanges
	SET Yield = 3
	WHERE (YieldType  = 'YIELD_GOLD' OR YieldType = 'YIELD_PRODUCTION') AND BuildingType = 'BUILDING_MM_DRUGSTORE_DUMMY' AND EXISTS (SELECT * FROM COMMUNITY WHERE Type = 'COMMUNITY_CORE_BALANCE_BUILDINGS' AND Value = 1 );

UPDATE Building_YieldModifiers
	SET Yield = 0
	WHERE YieldType = 'YIELD_GOLD' AND BuildingType = 'BUILDING_MM_DRUGSTORE' AND EXISTS (SELECT * FROM COMMUNITY WHERE Type = 'COMMUNITY_CORE_BALANCE_BUILDINGS' AND Value = 1 );

INSERT INTO Building_ResourceYieldChanges
	(BuildingType, ResourceType, YieldType, Yield)
VALUES
	('BUILDING_MM_DRUGSTORE', 'RESOURCE_SPICES', 'YIELD_FOOD', 
		(CASE WHEN EXISTS(SELECT * FROM COMMUNITY WHERE Type = 'COMMUNITY_CORE_BALANCE_BUILDINGS' AND Value = 1)
		THEN 1
		ELSE 0 END)),
	('BUILDING_MM_DRUGSTORE', 'RESOURCE_SPICES', 'YIELD_PRODUCTION',
		(CASE WHEN EXISTS(SELECT * FROM COMMUNITY WHERE Type = 'COMMUNITY_CORE_BALANCE_BUILDINGS' AND Value = 1)
		THEN 1
		ELSE 0 END)),
	('BUILDING_MM_DRUGSTORE', 'RESOURCE_SUGAR', 'YIELD_GOLD',
		(CASE WHEN EXISTS(SELECT * FROM COMMUNITY WHERE Type = 'COMMUNITY_CORE_BALANCE_BUILDINGS' AND Value = 1)
		THEN 1
		ELSE 0 END)),
	('BUILDING_MM_DRUGSTORE', 'RESOURCE_SUGAR', 'YIELD_FOOD',
		(CASE WHEN EXISTS(SELECT * FROM COMMUNITY WHERE Type = 'COMMUNITY_CORE_BALANCE_BUILDINGS' AND Value = 1)
		THEN 1
		ELSE 0 END));

INSERT INTO Building_YieldFromBirth
	(BuildingType, YieldType, Yield)
VALUES
	('BUILDING_MM_DRUGSTORE', 'YIELD_PRODUCTION', 15);

INSERT INTO Building_YieldFromInternalTREnd
	(BuildingType, YieldType, Yield)
VALUES
	('BUILDING_MM_BUS_STOP', 'YIELD_FOOD', 25);


--UPDATE UnitPromotions SET AttackFullyHealedMod = 50 WHERE Type = 'PROMOTION_MM_PSI_TELEPORT_II';
--UPDATE UnitPromotions SET ExtraAttacks = 1 WHERE Type = 'PROMOTION_MM_PSI_TELEPORT_I';
--UPDATE UnitPromotions SET ExtraAttacks = 1 WHERE Type = 'PROMOTION_MM_PSI_TELEPORT_II';
--UPDATE UnitPromotions SET CanMoveAfterAttacking = 1 WHERE Type = 'PROMOTION_MM_PSI_TELEPORT_I';
--UPDATE UnitPromotions SET CanMoveAfterAttacking = 1 WHERE Type = 'PROMOTION_MM_PSI_TELEPORT_II';
--UPDATE UnitPromotions SET Help = 'TXT_KEY_PROMOTION_MM_VP_PSI_TELEPORT_I_HELP' WHERE Type = 'PROMOTION_MM_PSI_TELEPORT_I';
--UPDATE UnitPromotions SET Help = 'TXT_KEY_PROMOTION_MM_VP_PSI_TELEPORT_II_HELP' WHERE Type = 'PROMOTION_MM_PSI_TELEPORT_II';
--UPDATE UnitPromotions SET AoEWhileFortified = 10 WHERE Type = 'PROMOTION_MM_PSI_SHIELD';


--UPDATE Buildings
--	SET FinsihLandTRTourism = 10
--	WHERE Type = 'BUILDING_MM_BUS_STOP' AND EXISTS (SELECT * FROM COMMUNITY WHERE Type = 'COMMUNITY_CORE_BALANCE_BUILDINGS' AND Value = 1 );



