--This file exists to help increase modularity with PSI techniques
--It allows other units to use predefined PSI, or (perhaps) even the creation
--of entirely new PSI!

------------------
--Table to define that a PSI technique exists
--	'Type': Unique identifier for each tech
--	'Self': Boolean; determines if the move can be used on the unit that called it
--	'Enemy': Boolean; determines if the move can be used on an enemy unit
--	'Friendly': Boolean; determines if the move can be used on a friendly unit
--These require further scripting in lua
------------------
CREATE TABLE IF NOT EXISTS MMPSI
	(Type		TEXT,				
	Self		BOOLEAN,		
	Enemy		BOOLEAN,			
	Friendly	BOOLEAN);
INSERT INTO MMPSI
	(Type,					Self,		Enemy,			Friendly)
VALUES
	('PSI_PK_FIRE',			0,			1,				0),
	('PSI_PK_FREEZE',		0,			1,				0),
	('PSI_PK_THUNDER',		0,			1,				0),
	('PSI_LIFEUP',			1,			0,				0),
	('PSI_PSI_SHIELD',		1,			0,				0),
	('PSI_HYPNOSIS',		0,			1,				0),
	('PSI_PK_BEAM',			0,			1,				0),
	('PSI_PK_STARSTORM',	0,			1,				0);

------------------------------------
--Table to define a group of PSI techniques
--The lua scripting behind these will handle the criteria for choosing what PSI technique is used
--	'Type': Unique identifier for each group
--	'PSI': The PSI Type, determines what PSI belongs to this type
--	'Weight': The chance, within this group, for the PSI to be used
-------------------------------------
CREATE TABLE IF NOT EXISTS MMPSIGroup
	(Type,				PSI,				Weight);
INSERT INTO MMPSIGroup
	(Type,				PSI,				Weight)
VALUES
	('MM_EAGLE',		'PSI_PK_FIRE',		15),
	('MM_EAGLE',		'PSI_PK_FREEZE',	15),
	('MM_EAGLE',		'PSI_PK_THUNDER',	15),
	('MM_EAGLE',		'PSI_LIFEUP',		15),
	('MM_EAGLE',		'PSI_PSI_SHIELD',	15),
	('MM_EAGLE',		'PSI_HYPNOSIS',		10),

	('STARMAN',			'PSI_PK_BEAM',		33),
	('STARMAN',			'PSI_PSI_SHIELD',	20),
	('STARMAN',			'PSI_LIFEUP',		25),
	('STARMAN',			'PSI_HYPNOSIS',		10),
	('STARMAN',			'PSI_PK_STARSTORM',	4);

--Table to define what PSIGroup a unit belongs to
CREATE TABLE IF NOT EXISTS MMUnit_PSIGroup
	(UnitType,				PSIGroup);
INSERT INTO MMUnit_PSIGroup
	(UnitType,				PSIGroup)
VALUES
	('UNIT_MM_PSYCHIC',		'MM_EAGLE'),
	('UNIT_STARMEN',		'STARMAN');

DELETE FROM MMUnit_PSIGroup
WHERE UnitType = 'UNIT_STARMEN' AND EXISTS(SELECT * FROM MMEagle_StarmanDX WHERE Type = 'MMSTARMAN_STARMEN_USE_PSI' and Value = 0);
